"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_extension_tester_1 = require("vscode-extension-tester");
const chai_1 = require("chai");
const fsPromises = require("fs/promises");
const os = require("os");
const path = require("path");
const child_process_1 = require("child_process");
const TestResources_1 = require("./TestResources");
describe.only("aboba", function () {
    this.timeout(40000);
    let view;
    let settingsView;
    let licensePath;
    // https://pvs-studio.myjetbrains.com/articles/id-A-430/UI-testy#vvod-korrektnoj-licenzii
    describe("Кейс: Ввод корректной лицензии", function () {
        it("Удалили старую лицензию", () => __awaiter(this, void 0, void 0, function* () {
            yield prepareForTests(view);
            let pvsFolderPath;
            if (os.platform() === "win32") {
                const appDataDir = process.env.APPDATA;
                pvsFolderPath = path.join(appDataDir, "PVS-Studio");
            }
            else {
                const homeDir = os.homedir();
                pvsFolderPath = path.join(homeDir, ".config", "PVS-Studio");
            }
            licensePath = path.join(pvsFolderPath, "PVS-Studio.lic");
            yield removePath(licensePath);
            const isLicenseExists = yield isPathExists(licensePath);
            (0, chai_1.expect)(isLicenseExists).equal(false);
        }));
        it("Открыли панель настроек", () => __awaiter(this, void 0, void 0, function* () {
            yield new vscode_extension_tester_1.Workbench().executeCommand("PVS-Studio: Show settings");
            // Задержка, тк иначе VSCode не успеет понять, что появился новый таб
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(500);
            settingsView = new vscode_extension_tester_1.WebView();
            yield settingsView.switchToFrame();
            const settingsTabsElem = yield settingsView.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="settings-tabs"]`));
            (0, chai_1.expect)(settingsTabsElem).not.equal(undefined);
        }));
        it("Тыкнули на таб License и открыли настройки лицензии", () => __awaiter(this, void 0, void 0, function* () {
            const licenseTabElem = yield settingsView.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="settings-tab-license"]`));
            yield licenseTabElem.click();
            const licenseViewElem = yield settingsView.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="settings-license-view"]`));
            (0, chai_1.expect)(licenseViewElem).not.equal(undefined);
        }));
        it("Проверили, что поля лицензии пустые", () => __awaiter(this, void 0, void 0, function* () {
            const licenseViewElems = yield getLicenseViewElements(settingsView);
            const nameValue = yield licenseViewElems.nameInputElem.getAttribute("value");
            const keyValue = yield licenseViewElems.keyInputElem.getAttribute("value");
            const statusValue = yield licenseViewElems.statusTextElem.getText();
            (0, chai_1.expect)(nameValue).equal("");
            (0, chai_1.expect)(keyValue).equal("");
            (0, chai_1.expect)(statusValue).equal("");
        }));
        it("Заполнили лицензию и нажали на Activate", () => __awaiter(this, void 0, void 0, function* () {
            const licenseViewElems = yield getLicenseViewElements(settingsView);
            const licenseInfo = (yield fsPromises.readFile(TestResources_1.testResources.licenseInfo)).toString();
            const [name, key] = licenseInfo.split("\n");
            yield licenseViewElems.nameInputElem.sendKeys(name);
            yield licenseViewElems.keyInputElem.sendKeys(key);
            yield licenseViewElems.submitBtnElem.click();
            (0, chai_1.expect)(yield licenseViewElems.nameInputElem.getAttribute("value")).equal(name);
            (0, chai_1.expect)(yield licenseViewElems.keyInputElem.getAttribute("value")).equal(key);
        }));
        it("Получили всплывашку что лицензия активирована", () => __awaiter(this, void 0, void 0, function* () {
            yield settingsView.switchBack();
            // Да, снова можем не отработать нормально, нужна задержка
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(1000);
            const licenseValidNotification = yield getNotificationWithMessage("License state: Valid thru");
            (0, chai_1.expect)(licenseValidNotification).not.equal(undefined);
        }));
        it("Появился файл лицензии", () => __awaiter(this, void 0, void 0, function* () {
            const isLicenseFileExists = yield isPathExists(licensePath);
            (0, chai_1.expect)(isLicenseFileExists).equal(true);
        }));
    });
    // https://pvs-studio.myjetbrains.com/articles/id-A-430/UI-testy#otkrytie-proekta-i-zapusk-analiza
    describe("Кейс: Открытие проекта и запуск анализа", function () {
        it("Открыли тестовый проект", () => __awaiter(this, void 0, void 0, function* () {
            yield prepareForTests(view);
            yield vscode_extension_tester_1.VSBrowser.instance.openResources(TestResources_1.testResources.openProjectRunAnalysis);
            yield removePath(path.join(TestResources_1.testResources.openProjectRunAnalysis, ".PVS-Studio", "State"));
            const explorerViewControl = yield new vscode_extension_tester_1.ActivityBar().getViewControl("Explorer");
            (0, chai_1.expect)(explorerViewControl).not.equal(undefined);
        }));
        it("Тыкнули на кнопку анализа", () => __awaiter(this, void 0, void 0, function* () {
            yield new vscode_extension_tester_1.Workbench().executeCommand("PVS-Studio: Show Window");
            // задержка нужна, иначе селениум упадет
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(4000);
            view = new vscode_extension_tester_1.WebviewView();
            yield view.switchToFrame();
            const startAnalysisBtn = yield view.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="btn-start-analysis"]`));
            yield startAnalysisBtn.click();
            (0, chai_1.expect)(startAnalysisBtn).not.equal(undefined);
        }));
        it("Выбрали обычный анализ", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchBack();
            const input = yield vscode_extension_tester_1.InputBox.create(10000);
            const picks = yield input.getQuickPicks();
            const pickLabels = yield Promise.all(picks.map((p) => __awaiter(this, void 0, void 0, function* () { return yield p.getLabel(); })));
            const regularAnalysisPickIndex = pickLabels.findIndex((p) => p.includes("Regular analysis"));
            const regularAnalysisPick = picks[regularAnalysisPickIndex];
            yield regularAnalysisPick.select();
        }));
        it("Получили сообщение о build папке", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchBack();
            const noBuildDirNotification = yield getNotificationWithMessage("build");
            (0, chai_1.expect)(noBuildDirNotification).not.equal(undefined);
        }));
        it("Сбилдили проект", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchBack();
            yield new vscode_extension_tester_1.Workbench().executeCommand("cmake.build");
            const input = yield vscode_extension_tester_1.InputBox.create(10000);
            const picks = yield input.getQuickPicks();
            // TODO выбрать определенный билдер
            yield picks[2].select();
            // TODO хз как проверить что проект сбилдили
            (0, chai_1.expect)(picks.length).above(0);
        }));
        it("Тыкнули на кнопку анализа", () => __awaiter(this, void 0, void 0, function* () {
            // задержка нужна, тк хз сколько длится билд
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(8000);
            yield new vscode_extension_tester_1.Workbench().executeCommand("PVS-Studio: Show Window");
            // задержка нужна, иначе селениум упадет
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(4000);
            view = new vscode_extension_tester_1.WebviewView();
            yield view.switchToFrame();
            const startAnalysisBtn = yield view.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="btn-start-analysis"]`));
            yield startAnalysisBtn.click();
            (0, chai_1.expect)(startAnalysisBtn).not.equal(undefined);
        }));
        it("Выбрали обычный анализ", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchBack();
            const input = yield vscode_extension_tester_1.InputBox.create(10000);
            const picks = yield input.getQuickPicks();
            const pickLabels = yield Promise.all(picks.map((p) => __awaiter(this, void 0, void 0, function* () { return yield p.getLabel(); })));
            const regularAnalysisPickIndex = pickLabels.findIndex((p) => p.includes("Regular analysis"));
            const regularAnalysisPick = picks[regularAnalysisPickIndex];
            yield regularAnalysisPick.select();
        }));
        it("Анализ прошел успешно", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchToFrame();
            // Когда кнопка Стопа сменится на Старт - анализ завершился
            yield vscode_extension_tester_1.VSBrowser.instance.driver.wait(vscode_extension_tester_1.until.elementLocated(vscode_extension_tester_1.By.css(`[data-test-id="btn-start-analysis"]`)));
            const rows = yield view.findWebElements(vscode_extension_tester_1.By.css(`[data-test-id="table-row"]`));
            (0, chai_1.expect)(rows.length).greaterThan(1);
        }));
    });
    // https://pvs-studio.myjetbrains.com/articles/id-A-430/UI-testy#pauza-i-ili-otmena-analiza
    describe("Кейс: Пауза и/или отмена анализа", function () {
        it("Открыли тестовый проект", () => __awaiter(this, void 0, void 0, function* () {
            yield prepareForTests(view);
            yield vscode_extension_tester_1.VSBrowser.instance.openResources(TestResources_1.testResources.stopAnalysis);
            const explorerViewControl = yield new vscode_extension_tester_1.ActivityBar().getViewControl("Explorer");
            (0, chai_1.expect)(explorerViewControl).not.equal(undefined);
        }));
        it("Сбилдили проект", () => __awaiter(this, void 0, void 0, function* () {
            // задержка нужна, иначе селениум упадет
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(4000);
            yield new vscode_extension_tester_1.Workbench().executeCommand("cmake.build");
            const input = yield vscode_extension_tester_1.InputBox.create();
            // Я хз почему await выше не отрабатывает, но итем надо люто ждать
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(8000);
            const picks = yield input.getQuickPicks();
            yield picks[2].select();
            // TODO хз как проверить что проект сбилдили
            (0, chai_1.expect)(picks.length).above(0);
        }));
        it("Запустили анализ и видим pvs процессы", () => __awaiter(this, void 0, void 0, function* () {
            // задержка нужна, тк хз сколько длится билд
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(8000);
            yield new vscode_extension_tester_1.Workbench().executeCommand("PVS-Studio: Show Window");
            // задержка нужна, иначе селениум упадет
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(4000);
            view = new vscode_extension_tester_1.WebviewView();
            yield view.switchToFrame();
            const startAnalysisBtn = yield view.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="btn-start-analysis"]`));
            yield startAnalysisBtn.click();
            yield view.switchBack();
            const input = yield vscode_extension_tester_1.InputBox.create();
            const picks = yield input.getQuickPicks();
            const pickLabels = yield Promise.all(picks.map((p) => __awaiter(this, void 0, void 0, function* () { return yield p.getLabel(); })));
            const regularAnalysisPickIndex = pickLabels.findIndex((p) => p.includes("Regular analysis"));
            const regularAnalysisPick = picks[regularAnalysisPickIndex];
            yield regularAnalysisPick.select();
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(1000);
            const processes = yield getProcessTree(process.pid);
            (0, chai_1.expect)(processes.includes("pvs-studio")).equal(true);
        }));
        it("Стопнули анализ и нет никаких pvs процессов", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchToFrame();
            const stopAnalysisBtn = yield vscode_extension_tester_1.VSBrowser.instance.driver.wait(vscode_extension_tester_1.until.elementLocated(vscode_extension_tester_1.By.css(`[data-test-id="btn-stop-analysis"]`)));
            yield stopAnalysisBtn.click();
            yield vscode_extension_tester_1.VSBrowser.instance.driver.wait(vscode_extension_tester_1.until.elementLocated(vscode_extension_tester_1.By.css(`[data-test-id="btn-start-analysis"]`)));
            const processesAfterAnalysis = yield getProcessTree(process.pid);
            (0, chai_1.expect)(processesAfterAnalysis.includes("pvs-studio")).equal(false);
        }));
    });
    // https://pvs-studio.myjetbrains.com/articles/id-A-430/UI-testy#zagruzka-nekorrektnogo-otchyota
    describe("Кейс: Загрузка некорректного отчёта", function () {
        it("Тыкнули на кнопку загрузки отчета", () => __awaiter(this, void 0, void 0, function* () {
            yield prepareForTests(view);
            yield new vscode_extension_tester_1.Workbench().executeCommand("PVS-Studio: Show Window");
            // задержка нужна, иначе селениум упадет
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(4000);
            view = new vscode_extension_tester_1.WebviewView();
            yield view.switchToFrame();
            const openReportBtn = yield view.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="btn-open-report"]`));
            yield openReportBtn.click();
            (0, chai_1.expect)(openReportBtn).not.equal(undefined);
        }));
        it("Указали путь к отчету", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchBack();
            const input = yield vscode_extension_tester_1.InputBox.create();
            yield input.setText(TestResources_1.testResources.openInvalidReport);
            yield input.confirm();
        }));
        it("Получили всплывашку о том, что проект невалидный", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchBack();
            const workbench = new vscode_extension_tester_1.Workbench();
            const validReportNotification = yield getNotificationWithMessage("Incorrect format of PVS-Studio report.");
            (0, chai_1.expect)(validReportNotification).not.equal(undefined);
        }));
    });
    // https://pvs-studio.myjetbrains.com/articles/id-A-430/UI-testy#sohranenie-polnogo-otchyota
    describe("Кейс: Сохранение полного отчёта", function () {
        it("Тыкнули на кнопку загрузки отчета", () => __awaiter(this, void 0, void 0, function* () {
            yield prepareForTests(view);
            yield new vscode_extension_tester_1.Workbench().executeCommand("PVS-Studio: Show Window");
            // задержка нужна, иначе селениум упадет
            yield vscode_extension_tester_1.VSBrowser.instance.driver.sleep(4000);
            view = new vscode_extension_tester_1.WebviewView();
            yield view.switchToFrame();
            const openReportBtn = yield view.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="btn-open-report"]`));
            yield openReportBtn.click();
            (0, chai_1.expect)(openReportBtn).not.equal(undefined);
        }));
        it("Указали путь к отчету и собсна открыли отчет", () => __awaiter(this, void 0, void 0, function* () {
            yield view.switchBack();
            const input = yield vscode_extension_tester_1.InputBox.create();
            yield input.setText(TestResources_1.testResources.fullSaveReport);
            yield input.confirm();
            yield view.switchToFrame();
            const tableElem = yield view.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="table"]`));
            (0, chai_1.expect)(tableElem).not.equal(undefined);
        }));
        it("Тыкнули на кнопку Сохранить и указали путь куда, получили сообщение об успехе", () => __awaiter(this, void 0, void 0, function* () {
            const btnSaveElem = yield view.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="btn-save-report"]`));
            yield btnSaveElem.click();
            yield view.switchBack();
            const input = yield vscode_extension_tester_1.InputBox.create();
            yield input.setText(TestResources_1.testResources.fullSaveReportFromPlugin);
            yield input.confirm();
            const reportSavedNotification = yield getNotificationWithMessage("Report has been saved");
            (0, chai_1.expect)(reportSavedNotification).not.equal(undefined);
        }));
        it("Сравнили количество предупреждений в двух файлах, и оно одинаковое!", () => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            const file1Content = yield fsPromises.readFile(TestResources_1.testResources.fullSaveReport, "utf8");
            const file2Content = yield fsPromises.readFile(TestResources_1.testResources.fullSaveReportFromPlugin, "utf8");
            const file1Json = JSON.parse(file1Content);
            const file2Json = JSON.parse(file2Content);
            const file1Warnings = (_a = file1Json.warnings) !== null && _a !== void 0 ? _a : [];
            const file2Warnings = (_b = file2Json.warnings) !== null && _b !== void 0 ? _b : [];
            (0, chai_1.expect)(file1Warnings.length).eq(file2Warnings.length);
        }));
    });
});
function prepareForTests(view) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log("Ща подготовим всё для следующего теста...");
        if (view) {
            view.switchBack();
        }
        const workbench = new vscode_extension_tester_1.Workbench();
        const notificationsCenter = yield workbench.openNotificationsCenter();
        notificationsCenter.clearAllNotifications();
        const editorView = new vscode_extension_tester_1.EditorView();
        yield editorView.closeAllEditors();
        yield removePath(path.join(TestResources_1.testResources.openProjectRunAnalysis, ".PVS-Studio"));
        yield removePath(path.join(TestResources_1.testResources.openProjectRunAnalysis, "build"));
        yield removePath(path.join(TestResources_1.testResources.stopAnalysis, ".PVS-Studio"));
        yield removePath(path.join(TestResources_1.testResources.stopAnalysis, "build"));
        yield removePath(TestResources_1.testResources.fullSaveReportFromPlugin);
        yield new vscode_extension_tester_1.Workbench().executeCommand("Developer: Reload Webviews");
        console.log("Подготовили среду для теста.");
    });
}
function removePath(path) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield fsPromises.rm(path, { recursive: true, force: true });
        }
        catch (error) {
            console.error(`Error removing folder at ${path}:`, error);
        }
    });
}
function isPathExists(path) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield fsPromises.access(path);
            return true;
        }
        catch (_a) {
            return false;
        }
    });
}
function getLicenseViewElements(settingsView) {
    return __awaiter(this, void 0, void 0, function* () {
        const nameInputElem = yield settingsView.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="settings-license-name-input"]`));
        const keyInputElem = yield settingsView.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="settings-license-key-input"]`));
        const statusTextElem = yield settingsView.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="settings-license-status-text"]`));
        const submitBtnElem = yield settingsView.findWebElement(vscode_extension_tester_1.By.css(`[data-test-id="settings-license-submit-btn"`));
        return {
            nameInputElem,
            keyInputElem,
            statusTextElem,
            submitBtnElem,
        };
    });
}
function getNotificationWithMessage(messageToFind) {
    return __awaiter(this, void 0, void 0, function* () {
        const workbench = new vscode_extension_tester_1.Workbench();
        const notifications = yield workbench.getNotifications();
        for (const notification of notifications) {
            const message = yield notification.getMessage();
            if (message.includes(messageToFind)) {
                return notification;
            }
        }
        return undefined;
    });
}
/**
 * Возвращает строку такого вида:
 *
 * code(45461)─┬─code(45464)
 *
            ├─code(45466)─┬─code(45516)

            │             ├─code(115646)

            │             └─code(893260)
 * @param pid идентификатор процесса
 * @returns
 */
function getProcessTree(pid) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise((resolve, reject) => {
            (0, child_process_1.exec)(`pstree -p ${pid} -T`, (error, stdout, stderr) => {
                if (error) {
                    reject(error);
                }
                if (stderr) {
                    reject(stderr);
                }
                resolve(stdout);
            });
        });
    });
}
//# sourceMappingURL=test.js.map